// Bypasses issue with A_CheckForReload not working correctly in GZDoom and A_RailAttack being higher in Zandronum.
Class QThunderbolt : QZWeapon {
	int weaponFrame;
	Default {
		+WEAPON.BFG;
		Decal "BulletChip";
		Inventory.Icon "W_LIGHTN";
		Inventory.PickupMessage "$Q_WEAPON_THUNDERBOLT_PICKUP";
		Obituary "$Q_WEAPON_THUNDERBOLT_OBITUARY";
		Tag "$Q_WEAPON_THUNDERBOLT";
		Weapon.AmmoGive 15;
		Weapon.AmmoType "QCellAmmo";
		Weapon.AmmoUse 1;
		Weapon.SelectionOrder 0;
	}
	const PlayerAnnimation = 3;
	const RailDiff = 2;
	const RailDrift = 2;
	const RailDuration = 4;
	const RailSparsity = 1;
	const ShotDamage = 30;
	const ShotRange = 600;
	const Shots = 6;
	const ViewRecoil = 1;
	States {
		Spawn:
			QTHB Z -1 NoDelay ACS_NamedExecuteAlways("Spin");
			Stop;
		Select:
			QTHB A 0 A_SetUserVar("user_weapon_animation", PlayerAnnimation);
			Goto Super::Select;
		Deselect:
			QTHB A 0;
			Goto Super::Deselect;
		Ready:
			QTHB A 0 { invoker.weaponFrame = 0; }
			QTHB A 4 A_WeaponReady;
			Wait;
		Discharge:
			QTHB B 4 A_FireCustomMissile("QThunderboltExplosion");
			QTHB A 36;
			Goto Ready;
		Fire:
			QTHB # 0 A_WeaponReady(WRF_NOBOB|WRF_NOFIRE|WRF_DISABLESWITCH); // Prevent any weapon switching while firing.
			QTHB # 0 Offset(0, 32) A_GunFlash;
			QTHB # 0 A_PlaySound("weapons/lstart", CHAN_5);
//			QTHB # 0 A_PlaySound("weapons/lhit", CHAN_Weapon);
			QTHB # 0 ACS_NamedExecuteWithResult("Inaccuracy");
			QTHB # 0 A_JumpIf(WaterLevel >= 3, "Discharge");
			Goto FireSound;
		Hold:
			QTHB # 0 A_GunFlash;
			QTHB # 0 ACS_NamedExecuteWithResult("Inaccuracy");
			QTHB # 0 A_JumpIf(WaterLevel >= 3, "Discharge");
			QTHB # 0 A_CheckForReload(4, "FireNoSound");
			Goto FireSound;
		FireSound:
			QTHB # 0 A_PlaySound("weapons/lhit", CHAN_Weapon);
			QTHB # 0 A_FireBullets(0, 0, -1, ACS_NamedExecuteWithResult("CalculateDamage", ShotDamage, 1), "QThunderArcSound", FBF_NoFlash|FBF_NoRandom|FBF_NoRandomPuffZ, ShotRange);
			Goto FireBolt;
		FireNoSound:
			QTHB # 0 A_FireBullets(0, 0, -1, ACS_NamedExecuteWithResult("CalculateDamage", ShotDamage, 1), "QThunderArc", FBF_NoFlash|FBF_NoRandom|FBF_NoRandomPuffZ, ShotRange);
			Goto FireBolt;
		FireBolt:
			// Visual Rail.
			QTHB # 0 A_RailAttack(0, 0, 1, "", "DD DD FF", RGF_Silent|RGF_NoPiercing|RGF_FullBright, RailDiff, "Null", 0, 0, ShotRange, RailDuration, RailSparsity, RailDrift, "QThunderShaft", 0);
			QTHB # 0 ACS_NamedExecuteAlways("Recoil", 0, ViewRecoil);
			Goto ChooseFireFrame;
		ChooseFireFrame:
			QTHB # 0 A_JumpIf(invoker.weaponFrame % 4 == 0, "FireFrame0");
			QTHB # 0 A_JumpIf(invoker.weaponFrame % 4 == 1, "FireFrame1");
			QTHB # 0 A_JumpIf(invoker.weaponFrame % 4 == 2, "FireFrame2");
			QTHB # 1 A_JumpIf(invoker.weaponFrame % 4 == 3, "FireFrame3");
			Loop;
		FireFrame0:
			QTHB B 4;
			QTHB C 0;
			Goto EndFireFrame;
		FireFrame1:
			QTHB C 4;
			QTHB D 0;
			Goto EndFireFrame;
		FireFrame2:
			QTHB D 4;
			QTHB E 0;
			Goto EndFireFrame;
		FireFrame3:
			QTHB E 4;
			QTHB B 0;
			Goto EndFireFrame;
		EndFireFrame:
			QTHB # 0 { invoker.weaponFrame++; }
			QTHB # 0 A_Refire;
			QTHB # 4 A_WeaponReady(WRF_NOBOB); // Lerp back to A but still allow firing and deselecting;
			Goto Ready;
		Flash:
			QTHF # 0 A_JumpIf(invoker.weaponFrame % 4 == 0, "Flash0");
			QTHF # 0 A_JumpIf(invoker.weaponFrame % 4 == 1, "Flash1");
			QTHF # 0 A_JumpIf(invoker.weaponFrame % 4 == 2, "Flash2");
			QTHF # 1 A_JumpIf(invoker.weaponFrame % 4 == 3, "Flash3");
			Loop;
		Flash:
			QTHF # 0 A_CheckForReload(4, 1, 1);
			Goto Flash3;
			QTHF # 0 A_CheckForReload(3, 1, 1);
			Goto Flash2;
			QTHF # 0 A_CheckForReload(2, 1, 1);
			Goto Flash1;
			QTHF # 0;
			Goto Flash0;

		Flash0:
			QTHF B 4 Bright;
			QTHF C 4 Bright;
			QTHF A 0 Bright;
			Stop;
		Flash1:
			QTHF C 4 Bright;
			QTHF D 4 Bright;
			QTHF A 0 Bright;
			Stop;
		Flash2:
			QTHF D 4 Bright;
			QTHF E 4 Bright;
			QTHF A 0 Bright;
			Stop;
		Flash3:
			QTHF E 4 Bright;
			QTHF B 4 Bright;
			QTHF A 0 Bright;
			Stop;
	}
}